'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

var _immutabilityHelper = require('immutability-helper');

var _immutabilityHelper2 = _interopRequireDefault(_immutabilityHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultState = function defaultState() {
  return {
    lastFormName: '',
    lastActionType: '',
    data: {}
  };
};

var getDelta = function getDelta(type, name, data) {
  return {
    lastFormName: {
      $set: name
    },
    lastActionType: {
      $set: type
    },
    data: _defineProperty({}, name, {
      $set: data
    })
  };
};

var doUpdateAction = function doUpdateAction(state, action) {
  var payload = action.payload;


  var delta = getDelta(_constants2.default.UPDATE, payload.name, {
    formData: payload.formData,
    formMeta: payload.formMeta
  });

  return (0, _immutabilityHelper2.default)(state, delta);
};

var doResetAction = function doResetAction(state, action) {
  var payload = action.payload;

  var delta = getDelta(_constants2.default.RESET, payload.name, {});
  return (0, _immutabilityHelper2.default)(state, delta);
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultState();
  var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (action.type === _constants2.default.UPDATE) {
    return doUpdateAction(state, action);
  }

  if (action.type === _constants2.default.RESET) {
    return doResetAction(state, action);
  }

  return state;
}

exports.default = reducer;